package de.schatenseite.android.waldemar;

import static de.schatenseite.android.waldemar.WaldemarPreferences.PREFERENCES_UPDATED;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.View;
import android.widget.RemoteViews;
import de.schatenseite.android.datepreference.DatePreference;

public class WaldemarWidget extends AppWidgetProvider {
	public static final String MIDNIGHTLY_WIDGET_UPDATE = "MIDNIGHTLY_WIDGET_UPDATE";
	private static final double SECONDS_PER_DAY = 24 * 60 * 60;                                                                                                        
	private static final double MILLISECONDS_PER_DAY = SECONDS_PER_DAY * 1000;                                                                                         
	private static final int CURRENTLY_NOT_USED_REQUEST_CODE = 0;                                                                                                      
	private static final int SUSPICIOUS_FLAGS = 0;
	private static final boolean DEBUG = false;
	static AlarmManager myAlarmManager;
	static PendingIntent myPendingIntent;
	
	static long getMidnightInMillis() {
		Calendar calendar = new GregorianCalendar();
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
		return calendar.getTimeInMillis();
	}
	
	@Override
	public void onEnabled(Context context) {
		Intent intent = new Intent(MIDNIGHTLY_WIDGET_UPDATE);
		PendingIntent pendingIntent = PendingIntent.getBroadcast(context,
				CURRENTLY_NOT_USED_REQUEST_CODE, intent, SUSPICIOUS_FLAGS);
		AlarmManager alarmManager = (AlarmManager) context
				.getSystemService(Context.ALARM_SERVICE);
		long midnightInMillis = getMidnightInMillis();
		long interval = AlarmManager.INTERVAL_DAY;
		if (DEBUG) {
			interval = AlarmManager.INTERVAL_FIFTEEN_MINUTES;
		}
		alarmManager.setRepeating(AlarmManager.RTC, midnightInMillis, interval,
				pendingIntent);
		saveAlarmManager(alarmManager, pendingIntent);
	}

	@Override
	public void onDisabled(Context context) {
		myAlarmManager.cancel(myPendingIntent);
	}

	@Override
	public void onReceive(Context context, Intent intent) {
		if (MIDNIGHTLY_WIDGET_UPDATE.equals(intent.getAction()) || PREFERENCES_UPDATED.equals(intent.getAction())) {
			Bundle extras = intent.getExtras();
			if (extras != null) {
				AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(context);
				ComponentName thisAppWidget = new ComponentName(
						context.getPackageName(),
						WaldemarWidget.class.getName());
				int[] appWidgetIds = appWidgetManager.getAppWidgetIds(thisAppWidget);

				onUpdate(context, appWidgetManager, appWidgetIds);
			}
		} else {
			super.onReceive(context, intent);
		}
	}

	@Override
	public void onUpdate(Context context, AppWidgetManager appWidgetManager,
			int[] appWidgetIds) {
		for (int appWidgetId : appWidgetIds) {
			updateAppWidget(context, appWidgetManager, appWidgetId);
		}
	}

	public static void updateAppWidget(Context context,
			AppWidgetManager appWidgetManager, int appWidgetId) {
		RemoteViews remoteViews = new RemoteViews(context.getPackageName(), R.layout.main);

		// Create an Intent to launch PreferenceActivity
        Intent intent = new Intent(context, WaldemarPreferences.class);
		PendingIntent pendingIntent = PendingIntent.getActivity(context,
				CURRENTLY_NOT_USED_REQUEST_CODE, intent, SUSPICIOUS_FLAGS);
        remoteViews.setOnClickPendingIntent(R.id.LinearLayout01, pendingIntent);

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
		remoteViews.setTextViewText(R.id.name, prefs.getString("pref_name", context.getString(R.string.title)));

		long timeNow = getMidnightInMillis();
		
		Integer mode = Integer.valueOf(prefs.getString("pref_mode", "1"));
		
		Calendar calStart = DatePreference.getDateFor(PreferenceManager.getDefaultSharedPreferences(context), "pref_dateStart"); 
	    long timeStart = calStart.getTimeInMillis();
	    
		Integer duration = Integer.parseInt(prefs.getString("pref_duration", context.getString(R.string.duration_default)));

		Calendar calThen = DatePreference.getDateFor(PreferenceManager.getDefaultSharedPreferences(context), "pref_dateThen"); 
	    long timeThen = calThen.getTimeInMillis();
	    
	    switch (mode) {
	    case 1: // only timeThen
			remoteViews.setViewVisibility(R.id.progressBarFrame, View.GONE);
			break;
	    case 2: // timeStart and timeThen
			duration = (int)Math.round((double)(timeThen - timeStart) / MILLISECONDS_PER_DAY);
			remoteViews.setViewVisibility(R.id.progressBarFrame, View.VISIBLE);
			break;
	    case 3: // duration and timeThen
			remoteViews.setViewVisibility(R.id.progressBarFrame, View.VISIBLE);
			break;
	    case 4: // timeStart and duration
			timeThen = timeStart + duration * (long)MILLISECONDS_PER_DAY;
			remoteViews.setViewVisibility(R.id.progressBarFrame, View.VISIBLE);
			break;
	    }

		Integer days = (int)Math.round((double)(timeThen - timeNow) / MILLISECONDS_PER_DAY);
		remoteViews.setTextViewText(R.id.daycount, String.valueOf(days));

		Integer percent = (int)(((double)(duration - days) / (double)duration)*100);
		String status = String.valueOf(duration - days)+"/"+String.valueOf(duration)+", "+String.valueOf(percent)+"%"; 
		
		if (mode == 1) {
			// workaround since we cannot make the progressbar invisible (look above)
			duration = days;
			status = "";
		}
		
		if (DEBUG) {
			DateFormat format = SimpleDateFormat.getTimeInstance(SimpleDateFormat.MEDIUM, Locale.getDefault());
			status = status + "\n" + format.format(new Date());
		}
		
		remoteViews.setProgressBar(R.id.progress, duration, duration - days, false);
		remoteViews.setTextViewText(R.id.status, status);

		appWidgetManager.updateAppWidget(appWidgetId, remoteViews);
	}

	private void saveAlarmManager(AlarmManager alarmManager,
			PendingIntent pendingIntent) {
		myAlarmManager = alarmManager;
		myPendingIntent = pendingIntent;
	}
}